package com.cesys.cebo.cebomsr;

import java.io.IOException;

import com.sun.jna.Native;
import com.sun.jna.Platform;

/**
 * Direct access to the CeboMsr-Base-API using JNA. Internal use only.
 *
 * @author Thomas Hoppe
 */
final class SystemConnector {
	private static boolean _logInfo = true;

	private SystemConnector() {
	}

	private static String formatSystemSpecific(String libName) {
		if (Platform.isWindows()) {
			return libName + ".dll";
		}
		if (Platform.isLinux()) {
			return "lib" + libName + ".so";
		}
		if (Platform.isMac()) {
			return "lib" + libName + ".dylib";
		}
		return libName;
	}

	private static String getLibraryPath() {
		String arch = System.getProperty("os.arch");
		String library;
		if (arch.contains("arm")) {
			library = "cebomsr-1.3-arm";
		} else
			library = arch.contains("64") ? "cebomsr-1.3-x64" : "cebomsr-1.3-x86";

		return library;
	}

	public static boolean isAvailable() {
		return true; //return new File(formatSystemSpecific(getLibraryPath())).exists();
	}

	static {
		//if (isAvailable())
		Native.register(getLibraryPath());
	}

	public static final int CeboMsrTypeAll = 0xceb00000;
	public static final int CeboMsrTypeUsb = 0xceb00001;
	public static final int CeboMsrTypeCeboLC = 0xceb00010;
	public static final int CeboMsrTypeCeboStick = 0xceb00011;

	public static final int MaxIdentLength = 1024;

	public static final int CeboParameterSingleEndedCount = 0;
	public static final int CeboParameterDifferentialCount = 1;
	public static final int CeboParameterAnalogOutputCount = 2;
	public static final int CeboParameterDigitalPortCount = 3;
	public static final int CeboParameterCounterCount = 4;
	public static final int CeboParameterTriggerCount = 5;
	public static final int CeboParameterMaxInputFrames = 6;
	public static final int CeboParameterCurrentSourceCount = 7;
	public static final int CeboParameterLedCount = 8;

	public static final int CeboMsrIoIdDigitalPort0 = 0xceb01000;
	public static final int CeboMsrIoIdDigitalPort1 = 0xceb01001;
	public static final int CeboMsrIoIdDigitalPort2 = 0xceb01002;
	public static final int CeboMsrIoIdDigitalPort3 = 0xceb01003;
	public static final int CeboMsrIoIdDigitalPort4 = 0xceb01004;
	public static final int CeboMsrIoIdDigitalPort5 = 0xceb01005;
	public static final int CeboMsrIoIdDigitalPort6 = 0xceb01006;
	public static final int CeboMsrIoIdDigitalPort7 = 0xceb01007;

	public static final int CeboMsrIoIdSingleEnded0 = 0xceb01100;
	public static final int CeboMsrIoIdSingleEnded1 = 0xceb01101;
	public static final int CeboMsrIoIdSingleEnded2 = 0xceb01102;
	public static final int CeboMsrIoIdSingleEnded3 = 0xceb01103;
	public static final int CeboMsrIoIdSingleEnded4 = 0xceb01104;
	public static final int CeboMsrIoIdSingleEnded5 = 0xceb01105;
	public static final int CeboMsrIoIdSingleEnded6 = 0xceb01106;
	public static final int CeboMsrIoIdSingleEnded7 = 0xceb01107;
	public static final int CeboMsrIoIdSingleEnded8 = 0xceb01108;
	public static final int CeboMsrIoIdSingleEnded9 = 0xceb01109;
	public static final int CeboMsrIoIdSingleEnded10 = 0xceb0110a;
	public static final int CeboMsrIoIdSingleEnded11 = 0xceb0110b;
	public static final int CeboMsrIoIdSingleEnded12 = 0xceb0110c;
	public static final int CeboMsrIoIdSingleEnded13 = 0xceb0110d;
	public static final int CeboMsrIoIdSingleEnded14 = 0xceb0110e;
	public static final int CeboMsrIoIdSingleEnded15 = 0xceb0110f;

	public static final int CeboMsrIoIdDifferential0 = 0xceb01200;
	public static final int CeboMsrIoIdDifferential1 = 0xceb01201;
	public static final int CeboMsrIoIdDifferential2 = 0xceb01202;
	public static final int CeboMsrIoIdDifferential3 = 0xceb01203;
	public static final int CeboMsrIoIdDifferential4 = 0xceb01204;
	public static final int CeboMsrIoIdDifferential5 = 0xceb01205;
	public static final int CeboMsrIoIdDifferential6 = 0xceb01206;
	public static final int CeboMsrIoIdDifferential7 = 0xceb01207;

	public static final int CeboMsrIoIdAnalogOutput0 = 0xceb01300;
	public static final int CeboMsrIoIdAnalogOutput1 = 0xceb01301;

	public static final int CeboMsrRange0 = 0xceb01400;
	public static final int CeboMsrRange1 = 0xceb01401;
	public static final int CeboMsrRange2 = 0xceb01402;
	public static final int CeboMsrRange3 = 0xceb01403;
	public static final int CeboMsrRangeInvalid = 0xceb014ff;

	public static final int CeboMsrIoIdCounter0 = 0xceb01500;

	public static final int CeboMsrIoIdTrigger0 = 0xceb01600;

	public static final int CeboMsrCurrentSource0 = 0xceb01700;
	public static final int CeboMsrCurrentSource1 = 0xceb01701;

	public static final int CeboMsrLedIndicator = 0xceb01800;

	public static final int CeboMsrPeripheralResetMaskInputFifo = 0x00000001;

	public static final int CeboMsrTriggerOutputPulse = 0xceb01900;
	public static final int CeboMsrTriggerOutputAlternating = 0xceb01901;
	public static final int CeboMsrTriggerInputRisingEdge = 0xceb01902;
	public static final int CeboMsrTriggerInputFallingEdge = 0xceb01903;
	public static final int CeboMsrTriggerInputAlternating = 0xceb01904;

	public static final int CeboMsrCounterConfigRisingEdge = 0xceb01a00;
	public static final int CeboMsrCounterConfigFallingEdge = 0xceb01a01;
	public static final int CeboMsrCounterConfigAlternating = 0xceb01a02;

	/**
	 * Check return code of library calls. If an error is reported, an Exception with the library error text is thrown.
	 * @param errorCode Code reported by the C-API.
	 * @throws IllegalArgumentException If the underlying API detects an illegal argument.
	 * @throws IndexOutOfBoundsException If the underlying API was forced to access an invalid array element.
	 * @throws IllegalStateException If a method is called that should not be called in the current state.
	 * @throws IOException Unrecoverable error in the lower API levels, in most cases an unpredictable communication error with the hardware.
	 * @throws RuntimeException If the API reports an unknown error, this should never be occur except for the case of a bug inside the API.
	 */
	public static void checkError(int errorCode) throws IOException {
		if (0 != errorCode) {
			// This is an error, get size of buffer for error string.
			int len = CeboMsrGetErrorLength(errorCode);

			// Retrieve error string.
			byte[] buffer = new byte[len];
			CeboMsrGetError(errorCode, buffer);

			String error = new String(buffer);

			int errorClass = (int)((long)(errorCode & 0xffffffffL) >> 24);

			switch (errorClass) {
			case 0x80: throw new IllegalArgumentException(error);
			case 0x40: throw new IndexOutOfBoundsException(error);
			case 0x20: throw new IllegalStateException(error);
			case 0x10: throw new IOException(error);
			}

			// Throw exception with string.
			throw new RuntimeException("(internal) Unhandled error: " + error);
		}
	}

	/// Error handling.
	public static native int CeboMsrGetErrorLength(int index);
	public static native void CeboMsrGetError(int index, byte[] buffer);

	/// Host version information.
	public static native int CeboMsrGetUsbBaseVersion(byte[] buffer);
	public static native int CeboMsrGetApiVersion(byte[] buffer);

	/// Enumeration.
	public static native int CeboMsrBeginEnumerate(int[] count, int msrType);
	public static native int CeboMsrBeginEnumerateNetwork(String hostname, int port, int[] count, int msrType);
	public static native int CeboMsrGetIdentifier(int id, byte[] ident, int[] msrType, byte[] version);

	/// Access.
	public static native int CeboMsrOpen(byte[] ident, int[] handle);
	public static native int CeboMsrClose(int handle);
	public static native int CeboMsrResetDevice(int handle);
	public static native int CeboMsrResetPeripherals(int handle, int mask);

	/// Box specific.
	public static native int CeboMsrGetDeviceParameter(int handle, int id, int[] value);
	public static native int CeboMsrGetDigitalPortIoCount(int handle, int ioId, int[] ioCount);
	public static native int CeboMsrGetPeripheralName(int handle, int id, byte[] name);
	public static native int CeboMsrGetDigitalPortIoName(int handle, int ioId, int io, byte[] name);
	public static native int CeboMsrEnumerateRanges(int handle, int ioId, int id, int[] rangeId);
	public static native int CeboMsrGetRangeMinMax(int handle, int ioId, int rangeId, float[] minValue, float[] maxValue);
	public static native int CeboMsrGetDefaultAnalogInputInterChannelDelay(int handle, int ioId, int rangeId, int[] defaultInterChannelDelay);
	public static native int CeboMsrGetMinAnalogInputInterChannelDelay(int handle, int ioId, int[] minInterChannelDelay);
	public static native int CeboMsrGetSerialNumber(int handle, byte[] buffer);
	public static native int CeboMsrGetTemperature(int handle, float[] temperature);
	public static native int CeboMsrSetLed(int handle, int led, int state);
	public static native int CeboMsrGetReferenceCurrent(int handle, int source, float[] current);

	/// IO specific.
	public static native int CeboMsrSetDigitalPortOutputEnableMask(int handle, int ioId, int mask);
	public static native int CeboMsrSetAnalogInputParameters(int handle, int ioId, int rangeId, int interChannelDelay, int[] correctedInterChannelDelay);
	public static native int CeboMsrGetAnalogInputParameters(int handle, int ioId, int[] rangeId, int[] interChannelDelay);
	public static native int CeboMsrSetTriggerConfig(int handle, int trigger, int triggerConfigId);
	public static native int CeboMsrGetTriggerConfig(int handle, int trigger, int[] triggerConfigId);
	public static native int CeboMsrSetTriggerEnable(int handle, int trigger, int enable);
	public static native int CeboMsrGetTriggerEnable(int handle, int trigger, int[] enable);
	public static native int CeboMsrResetCounter(int handle, int counter);
	public static native int CeboMsrSetCounterConfig(int handle, int counter, int counterConfigId);
	public static native int CeboMsrGetCounterConfig(int handle, int counter, int[] counterConfigId);
	public static native int CeboMsrSetCounterEnable(int handle, int counter, int enable);
	public static native int CeboMsrGetCounterEnable(int handle, int counter, int[] enable);

	/// Input specific.
	public static native int CeboMsrCalculateMaxBufferedInputFrames(int handle, int[] frameCount);
	public static native int CeboMsrSetupInputFrame(int handle, int[] ios, int count);
	public static native int CeboMsrStartDataAcquisition(int handle, int externalStarted, float frameRate, int frameCount, float[] correctedFrameRate);
	public static native int CeboMsrStopDataAcquisition(int handle);
	public static native int CeboMsrReadBlocking(int handle, float[] vbuffer, int[] buffer, int frameCount);
	public static native int CeboMsrReadNonBlocking(int handle, float[] vbuffer, int[] buffer, int[] transferred);
	public static native int CeboMsrReadDigitalPort(int handle, int ioId, int[] value);
	public static native int CeboMsrReadAnalogInput(int handle, int ioId, float[] value);
	public static native int CeboMsrReadFrame(int handle, float[] vbuffer, int[] buffer);
	public static native int CeboMsrReadCounter(int handle, int ioId, long[] value);

	/// Output specific.
	public static native int CeboMsrSetupOutputFrame(int handle, int[] ios, int count);
	public static native int CeboMsrWriteDigitalPort(int handle, int ioId, int value);
	public static native int CeboMsrWriteAnalogOutput(int handle, int ioId, float value);
	public static native int CeboMsrWriteFrame(int handle, float[] vbuffer, int[] buffer);

	/// Direct device access.
	public static native int __CeboMsrReadBlock(int handle, int address, byte[] buffer, int size);
	public static native int __CeboMsrWriteBlock(int handle, int address, byte[] buffer, int size);
	public static native int __CeboMsrReadRegister(int handle, int address, int[] value);
	public static native int __CeboMsrWriteRegister(int handle, int address, int value);
}
